//
//  Sample for VAD adjustment 
//

#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "init.h"
#include "screen.h"
#include "utility.h"

#include "sound.h"
#include "vct.h"

#include "vct_menu.h"
#include "demo_menu.h"


DWCInetControl stConnCtrl;

static u8 s_Work[ DWC_INIT_WORK_SIZE ] ATTRIBUTE_ALIGN( 32 );

static Menu *sMenu = NULL;
KeyControl stKeyCnt;
static VCTSession sSession[2];

static u8 stAudioBuffer[VCT_AUDIO_BUFFER_SIZE * VCT_DEFAULT_AUDIO_BUFFER_COUNT] ATTRIBUTE_ALIGN(32);

/*---------------------------------------------------------------------------*
   Initialize for menu of VAD 
 *---------------------------------------------------------------------------*/
static Menu* InitMenu()
{
    if (sMenu == NULL) {
        sMenu = new Menu(NULL);

        sMenu->AddItem(new VADInfo);
    }

    return sMenu;
}

/*---------------------------------------------------------------------------*
   Function of V-sync interrupts 
 *---------------------------------------------------------------------------*/
static void VBlankIntr(void)
{
    //---- upload pseudo screen to VRAM
    DC_FlushRange(gScreen, sizeof(gScreen));
    GX_LoadBG0Scr(gScreen, 0, sizeof(gScreen));
    
    OS_SetIrqCheckFlag(OS_IE_V_BLANK);
}

/*---------------------------------------------------------------------------*
   DWC alloc function 
 *---------------------------------------------------------------------------*/
static void*
AllocFunc( DWCAllocType name, u32   size, int align )
{
    void * ptr;
    OSIntrMode old;
    (void)name;
    (void)align;

    old = OS_DisableInterrupts();

    ptr = OS_AllocFromMain( size );

    OS_RestoreInterrupts( old );

    return ptr;
}

/*---------------------------------------------------------------------------*
   DWC free function 
 *---------------------------------------------------------------------------*/
static void
FreeFunc( DWCAllocType name, void* ptr,  u32 size  )
{
    OSIntrMode old;
    (void)name;
    (void)size;

    if ( !ptr ) return;

    old = OS_DisableInterrupts();

    OS_FreeToMain( ptr );

    OS_RestoreInterrupts( old );
}

/*---------------------------------------------------------------------------*
   Event callback function 
 *---------------------------------------------------------------------------*/
static void VoiceChatEventCallback(u8 aid, VCTEvent event, VCTSession *session, void *data)
{
#pragma unused(aid, data, event, session)
    // do nothing.
}

/*---------------------------------------------------------------------------*
   Main routine of VAD test 
 *---------------------------------------------------------------------------*/
static void VADTest()
{
    //
    //  Initialize libVCT 
    //
    VCTConfig           config;
    PMBackLightSwitch   sw = PM_BACKLIGHT_ON;
    
    config.mode            = VCT_MODE_PHONE;
    config.session         = sSession;
    config.numSession      = 2;
    config.aid             = NULL;
    config.callback        = VoiceChatEventCallback;
    config.userData        = NULL;
    config.audioBuffer     = stAudioBuffer;
    config.audioBufferSize = VCT_DEFAULT_AUDIO_BUFFER_COUNT * VCT_AUDIO_BUFFER_SIZE;

    SetupSound();

    if (!VCT_Init(&config)) {
        OS_TPrintf("ERROR: Can't initialize VoiceChat!!!\n");
    }

    for (;;) {

        ReadKeyData();
        int key = sMenu->ShowMenu();

        NNS_SndMain();
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);

        if (stKeyCnt.trg & PAD_BUTTON_SELECT) {
            break;
        }

        if (stKeyCnt.trg & PAD_BUTTON_DEBUG) {
            sw = (sw == PM_BACKLIGHT_ON) ? PM_BACKLIGHT_OFF : PM_BACKLIGHT_ON;
            (void)PM_SetBackLight( PM_LCD_ALL, sw);
        }
    }
    TerminateSound();

	return; 
}

/*---------------------------------------------------------------------------*
   Nitro main routine 
 *---------------------------------------------------------------------------*/
void NitroMain ()
{
    OS_Init();
    OS_InitTick();
    OS_InitAlarm();
    RTC_Init();

    PXI_Init();
    Heap_Init();
	Alarm_Init();

    GX_Init();
    FS_Init( MI_DMA_MAX_NUM ); 

    GX_DispOff();
    GXS_DispOff();


    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();
    (void)GX_VBlankIntr(TRUE);
	GX_SetDispSelect(GX_DISP_SELECT_SUB_MAIN);

    //  Initialize output screen 
    InitScreen();
  
    //  Displaying 
    OS_WaitIrq(TRUE, OS_IE_V_BLANK);              // Waiting the end of VBlank interrupt
    GX_DispOn();
    GXS_DispOn();

    InitKeyData(&stKeyCnt);
	InitMenu();

    //  Init NitroDWC 
    DWC_Init( s_Work );
    DWC_SetReportLevel(DWC_REPORTFLAG_ALL);
    DWC_SetMemFunc( AllocFunc, FreeFunc );

    VADTest();
    
	OS_Terminate();
}
