#include <nitro.h>

#include "vct.h"

#include "vct_menu.h"
#include "demo_util.h"
#include "main.h"
#include "thread_statistics.h"

//  Status of audio channels. 
BOOL gChannelStatus[3] = {FALSE, FALSE, FALSE};

extern GameControl stGameCnt;
//  for CPU usage statistics 
static u64 sAudioTick = 0;
static int sAudioCount = 0;

#pragma profile off

//
//  Name of libVCT state machine. 
//
static char * state_name[] = {
    "Init         ",
    "OutGoing     ",
    "Talking      ",
    "Connected    ",
    "Incoming     ",
    "Disconnecting",
};

//
//  Event name. 
//
static char *event_name[] = {
    "None",
    "Incoming",
    "Reject",
    "Busy",
    "Cancel",
    "Notify Free",
    "Notify Busy",
    "Connected",
    "Respondbye",
    "Disconnected",
    "Contact",
    "Timeout",
    "Abort",
};

const char * _demo_status_str[NUM_OF_DEMO_STATUS] = {
    "x",
    "ph",
    "tr",
    "conf",
    "talk",
    "?"
};

static char        sClientMsg[MAX_PLAYERS][32] = {
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
};

void AddAudioTick(u64 tick)
{
    sAudioTick += tick;
    ++sAudioCount;
}

const char* EventName(VCTEvent event)
{
    return event_name[event];
}

const char* StateName(VCTState state)
{
    return state_name[state];
    
}

void ShowTicks(const char *state_name)
{
    static OSTick prev = 0;
    static int flag = 0;
    static int count  = 0;
    OSTick now = OS_GetTick();

    u64 diff = now - prev;

    if (diff > OS_SecondsToTicks(1)) {
        if (flag) {
            PutMainScreen(31, 0, 4, "*");
        }
        else {
            PutMainScreen(31, 0, 4, " ");
        }
        flag = !flag;
        prev = now;

        //  display CPU usage of main routine and audio callback function. 
        u64 result[3];
        GetThreadTicks(result);
        for (int i = 0; i < 3; ++i) {
            result[i] = (result[i] * 1000) / diff;
            if (result[i] > 1000) {
                result[i] = 1000;
            }
        }

        sAudioTick = (sAudioCount) ? sAudioTick / sAudioCount : 0;
        
        PutMainScreen(8, 22, 18, "%4d / %2lld.%1lld%% %2lld.%1lld%% %2lld.%1lld%%",
                      (u32)sAudioTick,
                      result[0] / 10, result[0] % 10,
                      result[1] / 10, result[1] % 10,
                      result[2] / 10, result[2] % 10);
        sAudioTick = 0;
        sAudioCount = 0;
    }

    if (state_name) {
        PutMainScreen(1, 23, 15, (char*)state_name);
    }
    else {
        PutMainScreen(1, 23, 15, "-No session- ");
    }

    PutMainScreen( 0, 0, 15, "0:  1:  2:");
    PutMainScreen( 2, 0, FONT_COLOR_GREEN, "%c", gChannelStatus[0] ? 0x7b : ' ');
    PutMainScreen( 6, 0, FONT_COLOR_GREEN, "%c", gChannelStatus[1] ? 0x7b : ' ');
    PutMainScreen(10, 0, FONT_COLOR_GREEN, "%c", gChannelStatus[2] ? 0x7b : ' ');
}
static u8 sSendBuffer[512];

void ClearMenuList()
{
    for (int i = 0; i < MAX_PLAYERS; ++i) {
        sClientMsg[i][0] = '\0';
    }
}

void UpdateClientInfo(u8 aid, client_info *info)
{
    snprintf(sClientMsg[aid], 32, "aid: %d / pid: %d (%s)", aid, info->profileID,
             _demo_status_str[info->status]);
}

void UpdateMenuList(GameSequence *seq)
{
    for (int i = 0; i < MAX_PLAYERS; ++i) {
        seq->menuList[i].msg = (const char*)sClientMsg[i];
    }
}

//
//  Broadcast own status for all other clients. 
//
void BroadCastStatus(demo_status status)
{
    u8* pAidList;
    client_info info;
    int numHost;
    int i;

    info.magic     = DEMO_EXCHANGE_MAGIC_TOKEN;
    info.profileID = stGameCnt.userData.profileID;
    info.status    = status;

    numHost = DWC_GetAIDList(&pAidList);

    for (i = 0; i < numHost; i++){
        if (pAidList[i] == DWC_GetMyAID())
        {
            continue;
        }

        DWC_SendReliable(pAidList[i], &info, sizeof(info));
    }
}

/*---------------------------------------------------------------------------*
   DWC connection close callback function. 
 *---------------------------------------------------------------------------*/
void ConnectionClosedCallback(DWCError error,
                              BOOL isLocal,
                              BOOL isServer,
                              u8  aid,
                              int index,
                              void* param)
{

    if (error == DWC_ERROR_NONE){
        if (!isLocal && isServer){
            // When using server-client matchmaking and the local host is a client,
            // the server DS closed the connection.
            // Since the server DS has closed, the local host also closes.
            DWC_CloseConnectionsAsync();
        }
        
        if (DWC_GetNumConnectionHost() == 1){
            // When all connections are closed,
            if (isLocal){
                // Proceed if the connections with all members were closed 
                // when closed by the local host
                stGameCnt.blocking = FALSE;
            }
            if ((DWC_GetLastMatchingType() != DWC_MATCH_TYPE_SC_SV) || isServer){
                // if closed by the local host when isServer==TRUE,
                // or if closed by another host, set the next mode here
                *(GameMode *)param = GAME_MODE_LOGIN;
            }
            // When the self  is on the server-client matchmaking server DS,
            // do nothing if a client closes the connections of all members
        }

        if (isLocal){
            OS_TPrintf("Closed connection to aid %d (friendListIndex = %d) Rest %d.\n",
                       aid, index, DWC_GetNumConnectionHost());
        }
        else {
            OS_TPrintf("Connection to aid %d (friendListIndex = %d) was closed. Rest %d.\n",
                       aid, index, DWC_GetNumConnectionHost());
        }
    }
    else {
        DispErrorMessage();
        OS_TPrintf("Failed to close connections. %d\n\n", error);

        *(GameMode *)param = GAME_MODE_MAIN;

        stGameCnt.blocking = FALSE;
    }
}

